@@ -1,5 +1,6 @@
 extern crate proc_macro;
 
+use syn::parse::Parser;
 use syn::punctuated::Punctuated;
 use syn::spanned::Spanned;
 use syn::token::Comma;
@@ -109,7 +110,7 @@ fn add_lifetime_if_none(
 
 enum OpType {
     Regular,
-    Explicit(proc_macro2::Literal),
+    Explicit(OpTypeArgs),
     Implicit(OpTypeArgs),
 }
 
@@ -138,7 +139,7 @@ fn extract_field_properties(attrs: &[syn::Attribute]) -> (OpType, Option<syn::Li
     for attr in attrs {
         if attr.path.is_ident("explicit") {
             if let OpType::Regular = op_type {
-                op_type = OpType::Explicit(attr.parse_args::<proc_macro2::Literal>().unwrap());
+                op_type = OpType::Explicit(attr.parse_args::<OpTypeArgs>().unwrap());
             } else {
                 panic!("Can't specify #[explicit] or #[implicit] more than once")
             }
@@ -169,9 +170,18 @@ fn generate_read_element(
         .map_err(|e| e.add_location(asn1::ParseLocation::Field(#error_location)))
     };
     let mut read_op = match read_type {
-        OpType::Explicit(arg) => quote::quote! {
-            p.read_optional_explicit_element(#arg)#add_error_location?
-        },
+        OpType::Explicit(arg) => {
+            let value = arg.value;
+            if arg.required {
+                quote::quote! {
+                    p.read_explicit_element(#value)#add_error_location?
+                }
+            } else {
+                quote::quote! {
+                    p.read_optional_explicit_element(#value)#add_error_location?
+                }
+            }
+        }
         OpType::Implicit(arg) => {
             let value = arg.value;
             if arg.required {
@@ -275,7 +285,8 @@ fn generate_enum_read_block(
                     }
                 });
             }
-            OpType::Explicit(tag) => {
+            OpType::Explicit(arg) => {
+                let tag = arg.value;
                 read_blocks.push(quote::quote! {
                     if tlv.tag() == asn1::explicit_tag(#tag) {
                         return Ok(#name::#ident(asn1::parse(
@@ -331,9 +342,18 @@ fn generate_write_element(
     }
 
     match write_type {
-        OpType::Explicit(arg) => quote::quote_spanned! {f.span() =>
-            w.write_optional_explicit_element(#field_read, #arg);
-        },
+        OpType::Explicit(arg) => {
+            let value = arg.value;
+            if arg.required {
+                quote::quote_spanned! {f.span() =>
+                    w.write_explicit_element(#field_read, #value);
+                }
+            } else {
+                quote::quote_spanned! {f.span() =>
+                    w.write_optional_explicit_element(#field_read, #value);
+                }
+            }
+        }
         OpType::Implicit(arg) => {
             let value = arg.value;
             if arg.required {
@@ -400,7 +420,8 @@ fn generate_enum_write_block(name: &syn::Ident, data: &syn::DataEnum) -> proc_ma
                     #name::#ident(value) => w.write_element(value),
                 }
             }
-            OpType::Explicit(tag) => {
+            OpType::Explicit(arg) => {
+                let tag = arg.value;
                 quote::quote! {
                     #name::#ident(value) => w.write_optional_explicit_element(&Some(value), #tag),
                 }
@@ -419,3 +440,55 @@ fn generate_enum_write_block(name: &syn::Ident, data: &syn::DataEnum) -> proc_ma
         }
     }
 }
+
+// TODO: Duplicate of this function in src/object_identifier.rs, can we
+// de-dupe?
+fn _write_base128_int(data: &mut Vec<u8>, n: u32) {
+    if n == 0 {
+        data.push(0);
+        return;
+    }
+
+    let mut l = 0;
+    let mut i = n;
+    while i > 0 {
+        l += 1;
+        i >>= 7;
+    }
+
+    for i in (0..l).rev() {
+        let mut o = (n >> (i * 7)) as u8;
+        o &= 0x7f;
+        if i != 0 {
+            o |= 0x80;
+        }
+        data.push(o);
+    }
+}
+
+#[proc_macro]
+pub fn oid(item: proc_macro::TokenStream) -> proc_macro::TokenStream {
+    let p_arcs = Punctuated::<syn::LitInt, syn::Token![,]>::parse_terminated
+        .parse(item)
+        .unwrap();
+    let mut arcs = p_arcs.iter();
+
+    let mut der_encoded = vec![];
+    let first = arcs.next().unwrap().base10_parse::<u32>().unwrap();
+    let second = arcs.next().unwrap().base10_parse::<u32>().unwrap();
+    _write_base128_int(&mut der_encoded, 40 * first + second);
+    for arc in arcs {
+        _write_base128_int(&mut der_encoded, arc.base10_parse().unwrap());
+    }
+
+    let der_len = der_encoded.len();
+    // TODO: is there a way to use the `MAX_OID_LENGTH` constant here?
+    assert!(der_len <= 63);
+    der_encoded.resize(63, 0);
+    let der_lit = syn::LitByteStr::new(&der_encoded, proc_macro2::Span::call_site());
+    let expanded = quote::quote! {
+        asn1::ObjectIdentifier::from_der_unchecked(*#der_lit, #der_len as u8)
+    };
+
+    proc_macro::TokenStream::from(expanded)
+}
